/*   $Source: /var/local/cvs/upcr/upcr_geninclude/upcr_atomic_defs.h.in,v $
 *     $Date: 2010/07/16 17:49:06 $
 * $Revision: 1.1 $
 * Description:
 *  Berkeley UPC atomic extensions - external interface defines
 *
 * Copyright 2010,  E. O. Lawrence Berekely National Laboratory
 */

#ifndef UPCR_ATOMIC_DEFS_H
#define UPCR_ATOMIC_DEFS_H

/*---------------------------------------------------------------------------------*/
/* External interface: */

#if UPCRI_LIBWRAP
  #define _bupc_atomic_set_fixed(_type,_bits,_dst,_val,_isstrict) \
	_bupc_atomic##_bits##_set(_dst,(_type)(_val),_isstrict)
  #define _bupc_atomic_set_fixed_local(_type,_bits,_dst,_val) \
	_bupc_atomic##_bits##_set_local(_dst,(_type)(_val))

  #define _bupc_atomic_set_floating  ERROR__floating_point_atomics_unimplemented
  #define _bupc_atomic_set_floating_local  ERROR__floating_point_atomics_unimplemented

  #define _bupc_atomic_read_fixed(_type,_bits,_src,_isstrict) \
	((_type)_bupc_atomic##_bits##_read(_src,_isstrict))
  #define _bupc_atomic_read_fixed_local(_type,_bits,_src) \
	((_type)_bupc_atomic##_bits##_read_local(_src))

  #define _bupc_atomic_read_floating ERROR__floating_point_atomics_unimplemented
  #define _bupc_atomic_read_floating_local ERROR__floating_point_atomics_unimplemented

  #define _bupc_atomic_0op(_code,_operation,_dst,_isstrict) \
  	_bupc_atomic##_code##_##_operation(_dst,_isstrict)
  #define _bupc_atomic_0op_local(_code,_operation,_dst) \
  	_bupc_atomic##_code##_##_operation##_local(_dst,0)

  #define _bupc_atomic_1op(_code,_operation,_dst,_op1,_isstrict) \
  	_bupc_atomic##_code##_##_operation(_dst,_op1,_isstrict)
  #define _bupc_atomic_1op_local(_code,_operation,_dst,_op1) \
  	_bupc_atomic##_code##_##_operation##_local(_dst,_op1,0)

  #define _bupc_atomic_2op(_code,_operation,_dst,_op1,_op2,_isstrict) \
  	_bupc_atomic##_code##_##_operation(_dst,_op1,_op2,_isstrict)
  #define _bupc_atomic_2op_local(_code,_operation,_dst,_op1,_op2) \
  	_bupc_atomic##_code##_##_operation##_local(_dst,_op1,_op2,0)
#else
  #define _bupc_atomic_set_fixed(_type,_bits,_dst,_val,_isstrict) \
	(upcri_srcpos(),_bupc_atomic##_bits##_set(_dst,_val,_isstrict UPCRI_PT_PASS))
  #define _bupc_atomic_set_fixed_local(_type,_bits,_dst,_val) \
	(upcri_srcpos(),_bupc_atomic##_bits##_set_local(_dst,_val))

  #define _bupc_atomic_set_floating  ERROR__floating_point_atomics_unimplemented
  #define _bupc_atomic_set_floating_local  ERROR__floating_point_atomics_unimplemented

  #define _bupc_atomic_read_fixed(_type,_bits,_src,_isstrict) \
	(upcri_srcpos(),(_type)_bupc_atomic##_bits##_read(_src,_isstrict UPCRI_PT_PASS))
  #define _bupc_atomic_read_fixed_local(_type,_bits,_src) \
	(upcri_srcpos(),(_type)_bupc_atomic##_bits##_read_local(_src))

  #define _bupc_atomic_read_floating ERROR__floating_point_atomics_unimplemented
  #define _bupc_atomic_read_floating_local ERROR__floating_point_atomics_unimplemented

  #define _bupc_atomic_0op(_code,_operation,_dst,_isstrict) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation(_dst,_isstrict UPCRI_PT_PASS))
  #define _bupc_atomic_0op_local(_code,_operation,_dst) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation##_local(_dst,0))

  #define _bupc_atomic_1op(_code,_operation,_dst,_op1,_isstrict) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation(_dst,_op1,_isstrict UPCRI_PT_PASS))
  #define _bupc_atomic_1op_local(_code,_operation,_dst,_op1) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation##_local(_dst,_op1,0))

  #define _bupc_atomic_2op(_code,_operation,_dst,_op1,_op2,_isstrict) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation(_dst,_op1,_op2,_isstrict UPCRI_PT_PASS))
  #define _bupc_atomic_2op_local(_code,_operation,_dst,_op1,_op2) \
	(upcri_srcpos(),_bupc_atomic##_code##_##_operation##_local(_dst,_op1,_op2,0))
#endif

#if UPCRI_LIBWRAP
  #define _bupc_atomicU64_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(uint64_t,64,_dst,_val,0)
  #define _bupc_atomicI64_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(int64_t,64,_dst,_val,0)
  #define _bupc_atomicU32_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(uint32_t,32,_dst,_val,0)
  #define _bupc_atomicI32_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(int32_t,32,_dst,_val,0)
  #define _bupc_atomicU64_set_strict(_dst,_val)	_bupc_atomic_set_fixed(uint64_t,64,_dst,_val,1)
  #define _bupc_atomicI64_set_strict(_dst,_val)	_bupc_atomic_set_fixed(int64_t,64,_dst,_val,1)
  #define _bupc_atomicU32_set_strict(_dst,_val)	_bupc_atomic_set_fixed(uint32_t,32,_dst,_val,1)
  #define _bupc_atomicI32_set_strict(_dst,_val)	_bupc_atomic_set_fixed(int32_t,32,_dst,_val,1)
  #define _bupc_atomicU64_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(uint64_t,64,_dst,_val)
  #define _bupc_atomicI64_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(int64_t,64,_dst,_val)
  #define _bupc_atomicU32_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(uint32_t,32,_dst,_val)
  #define _bupc_atomicI32_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(int32_t,32,_dst,_val)
  #define _bupc_atomicU64_read_relaxed(_src)	_bupc_atomic_read_fixed(uint64_t,64,_src,0)
  #define _bupc_atomicI64_read_relaxed(_src)	_bupc_atomic_read_fixed(int64_t,64,_src,0)
  #define _bupc_atomicU32_read_relaxed(_src)	_bupc_atomic_read_fixed(uint32_t,32,_src,0)
  #define _bupc_atomicI32_read_relaxed(_src)	_bupc_atomic_read_fixed(int32_t,32,_src,0)
  #define _bupc_atomicU64_read_strict(_src)	_bupc_atomic_read_fixed(uint64_t,64,_src,1)
  #define _bupc_atomicI64_read_strict(_src)	_bupc_atomic_read_fixed(int64_t,64,_src,1)
  #define _bupc_atomicU32_read_strict(_src)	_bupc_atomic_read_fixed(uint32_t,32,_src,1)
  #define _bupc_atomicI32_read_strict(_src)	_bupc_atomic_read_fixed(int32_t,32,_src,1)
  #define _bupc_atomicU64_read_private(_src)	_bupc_atomic_read_fixed_local(uint64_t,64,_src)
  #define _bupc_atomicI64_read_private(_src)	_bupc_atomic_read_fixed_local(int64_t,64,_src)
  #define _bupc_atomicU32_read_private(_src)	_bupc_atomic_read_fixed_local(uint32_t,32,_src)
  #define _bupc_atomicI32_read_private(_src)	_bupc_atomic_read_fixed_local(int32_t,32,_src)
  #define _bupc_atomicU64_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchadd,_dst,_op,0)
  #define _bupc_atomicI64_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchadd,_dst,_op,0)
  #define _bupc_atomicU32_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchadd,_dst,_op,0)
  #define _bupc_atomicI32_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchadd,_dst,_op,0)
  #define _bupc_atomicU64_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchand,_dst,_op,0)
  #define _bupc_atomicI64_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchand,_dst,_op,0)
  #define _bupc_atomicU32_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchand,_dst,_op,0)
  #define _bupc_atomicI32_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchand,_dst,_op,0)
  #define _bupc_atomicU64_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchor,_dst,_op,0)
  #define _bupc_atomicI64_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchor,_dst,_op,0)
  #define _bupc_atomicU32_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchor,_dst,_op,0)
  #define _bupc_atomicI32_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchor,_dst,_op,0)
  #define _bupc_atomicU64_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchxor,_dst,_op,0)
  #define _bupc_atomicI64_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchxor,_dst,_op,0)
  #define _bupc_atomicU32_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchxor,_dst,_op,0)
  #define _bupc_atomicI32_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchxor,_dst,_op,0)
  #define _bupc_atomicU64_swap_relaxed(_dst,_op)	_bupc_atomic_1op(U64,swap,_dst,_op,0)
  #define _bupc_atomicI64_swap_relaxed(_dst,_op)	_bupc_atomic_1op(I64,swap,_dst,_op,0)
  #define _bupc_atomicU32_swap_relaxed(_dst,_op)	_bupc_atomic_1op(U32,swap,_dst,_op,0)
  #define _bupc_atomicI32_swap_relaxed(_dst,_op)	_bupc_atomic_1op(I32,swap,_dst,_op,0)
  #define _bupc_atomicU64_fetchnot_relaxed(_dst)	_bupc_atomic_0op(U64,fetchnot,_dst,0)
  #define _bupc_atomicI64_fetchnot_relaxed(_dst)	_bupc_atomic_0op(I64,fetchnot,_dst,0)
  #define _bupc_atomicU32_fetchnot_relaxed(_dst)	_bupc_atomic_0op(U32,fetchnot,_dst,0)
  #define _bupc_atomicI32_fetchnot_relaxed(_dst)	_bupc_atomic_0op(I32,fetchnot,_dst,0)
  #define _bupc_atomicU64_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U64,cswap,_dst,_op0,_op1,0)
  #define _bupc_atomicI64_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I64,cswap,_dst,_op0,_op1,0)
  #define _bupc_atomicU32_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U32,cswap,_dst,_op0,_op1,0)
  #define _bupc_atomicI32_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I32,cswap,_dst,_op0,_op1,0)
  #define _bupc_atomicU64_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U64,mswap,_dst,_op0,_op1,0)
  #define _bupc_atomicI64_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I64,mswap,_dst,_op0,_op1,0)
  #define _bupc_atomicU32_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U32,mswap,_dst,_op0,_op1,0)
  #define _bupc_atomicI32_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I32,mswap,_dst,_op0,_op1,0)
  #define _bupc_atomicU64_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchadd,_dst,_op,1)
  #define _bupc_atomicI64_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchadd,_dst,_op,1)
  #define _bupc_atomicU32_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchadd,_dst,_op,1)
  #define _bupc_atomicI32_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchadd,_dst,_op,1)
  #define _bupc_atomicU64_fetchand_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchand,_dst,_op,1)
  #define _bupc_atomicI64_fetchand_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchand,_dst,_op,1)
  #define _bupc_atomicU32_fetchand_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchand,_dst,_op,1)
  #define _bupc_atomicI32_fetchand_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchand,_dst,_op,1)
  #define _bupc_atomicU64_fetchor_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchor,_dst,_op,1)
  #define _bupc_atomicI64_fetchor_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchor,_dst,_op,1)
  #define _bupc_atomicU32_fetchor_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchor,_dst,_op,1)
  #define _bupc_atomicI32_fetchor_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchor,_dst,_op,1)
  #define _bupc_atomicU64_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchxor,_dst,_op,1)
  #define _bupc_atomicI64_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchxor,_dst,_op,1)
  #define _bupc_atomicU32_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchxor,_dst,_op,1)
  #define _bupc_atomicI32_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchxor,_dst,_op,1)
  #define _bupc_atomicU64_swap_strict(_dst,_op)	_bupc_atomic_1op(U64,swap,_dst,_op,1)
  #define _bupc_atomicI64_swap_strict(_dst,_op)	_bupc_atomic_1op(I64,swap,_dst,_op,1)
  #define _bupc_atomicU32_swap_strict(_dst,_op)	_bupc_atomic_1op(U32,swap,_dst,_op,1)
  #define _bupc_atomicI32_swap_strict(_dst,_op)	_bupc_atomic_1op(I32,swap,_dst,_op,1)
  #define _bupc_atomicU64_fetchnot_strict(_dst)	_bupc_atomic_0op(U64,fetchnot,_dst,1)
  #define _bupc_atomicI64_fetchnot_strict(_dst)	_bupc_atomic_0op(I64,fetchnot,_dst,1)
  #define _bupc_atomicU32_fetchnot_strict(_dst)	_bupc_atomic_0op(U32,fetchnot,_dst,1)
  #define _bupc_atomicI32_fetchnot_strict(_dst)	_bupc_atomic_0op(I32,fetchnot,_dst,1)
  #define _bupc_atomicU64_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U64,cswap,_dst,_op0,_op1,1)
  #define _bupc_atomicI64_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I64,cswap,_dst,_op0,_op1,1)
  #define _bupc_atomicU32_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U32,cswap,_dst,_op0,_op1,1)
  #define _bupc_atomicI32_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I32,cswap,_dst,_op0,_op1,1)
  #define _bupc_atomicU64_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U64,mswap,_dst,_op0,_op1,1)
  #define _bupc_atomicI64_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I64,mswap,_dst,_op0,_op1,1)
  #define _bupc_atomicU32_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U32,mswap,_dst,_op0,_op1,1)
  #define _bupc_atomicI32_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I32,mswap,_dst,_op0,_op1,1)
  #define _bupc_atomicU64_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchadd,_dst,_op)
  #define _bupc_atomicI64_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchadd,_dst,_op)
  #define _bupc_atomicU32_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchadd,_dst,_op)
  #define _bupc_atomicI32_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchadd,_dst,_op)
  #define _bupc_atomicU64_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchand,_dst,_op)
  #define _bupc_atomicI64_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchand,_dst,_op)
  #define _bupc_atomicU32_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchand,_dst,_op)
  #define _bupc_atomicI32_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchand,_dst,_op)
  #define _bupc_atomicU64_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchor,_dst,_op)
  #define _bupc_atomicI64_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchor,_dst,_op)
  #define _bupc_atomicU32_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchor,_dst,_op)
  #define _bupc_atomicI32_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchor,_dst,_op)
  #define _bupc_atomicU64_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchxor,_dst,_op)
  #define _bupc_atomicI64_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchxor,_dst,_op)
  #define _bupc_atomicU32_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchxor,_dst,_op)
  #define _bupc_atomicI32_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchxor,_dst,_op)
  #define _bupc_atomicU64_swap_private(_dst,_op)	_bupc_atomic_1op_local(U64,swap,_dst,_op)
  #define _bupc_atomicI64_swap_private(_dst,_op)	_bupc_atomic_1op_local(I64,swap,_dst,_op)
  #define _bupc_atomicU32_swap_private(_dst,_op)	_bupc_atomic_1op_local(U32,swap,_dst,_op)
  #define _bupc_atomicI32_swap_private(_dst,_op)	_bupc_atomic_1op_local(I32,swap,_dst,_op)
  #define _bupc_atomicU64_fetchnot_private(_dst)	_bupc_atomic_0op_local(U64,fetchnot,_dst)
  #define _bupc_atomicI64_fetchnot_private(_dst)	_bupc_atomic_0op_local(I64,fetchnot,_dst)
  #define _bupc_atomicU32_fetchnot_private(_dst)	_bupc_atomic_0op_local(U32,fetchnot,_dst)
  #define _bupc_atomicI32_fetchnot_private(_dst)	_bupc_atomic_0op_local(I32,fetchnot,_dst)
  #define _bupc_atomicU64_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U64,cswap,_dst,_op0,_op1)
  #define _bupc_atomicI64_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I64,cswap,_dst,_op0,_op1)
  #define _bupc_atomicU32_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U32,cswap,_dst,_op0,_op1)
  #define _bupc_atomicI32_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I32,cswap,_dst,_op0,_op1)
  #define _bupc_atomicU64_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U64,mswap,_dst,_op0,_op1)
  #define _bupc_atomicI64_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I64,mswap,_dst,_op0,_op1)
  #define _bupc_atomicU32_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U32,mswap,_dst,_op0,_op1)
  #define _bupc_atomicI32_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I32,mswap,_dst,_op0,_op1)
#else
  #define bupc_atomicU64_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(uint64_t,64,_dst,_val,0)
  #define bupc_atomicI64_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(int64_t,64,_dst,_val,0)
  #define bupc_atomicU32_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(uint32_t,32,_dst,_val,0)
  #define bupc_atomicI32_set_relaxed(_dst,_val)	_bupc_atomic_set_fixed(int32_t,32,_dst,_val,0)
  #define bupc_atomicU64_set_strict(_dst,_val)	_bupc_atomic_set_fixed(uint64_t,64,_dst,_val,1)
  #define bupc_atomicI64_set_strict(_dst,_val)	_bupc_atomic_set_fixed(int64_t,64,_dst,_val,1)
  #define bupc_atomicU32_set_strict(_dst,_val)	_bupc_atomic_set_fixed(uint32_t,32,_dst,_val,1)
  #define bupc_atomicI32_set_strict(_dst,_val)	_bupc_atomic_set_fixed(int32_t,32,_dst,_val,1)
  #define bupc_atomicU64_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(uint64_t,64,_dst,_val)
  #define bupc_atomicI64_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(int64_t,64,_dst,_val)
  #define bupc_atomicU32_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(uint32_t,32,_dst,_val)
  #define bupc_atomicI32_set_private(_dst,_val)	_bupc_atomic_set_fixed_local(int32_t,32,_dst,_val)
  #define bupc_atomicU64_read_relaxed(_src)	_bupc_atomic_read_fixed(uint64_t,64,_src,0)
  #define bupc_atomicI64_read_relaxed(_src)	_bupc_atomic_read_fixed(int64_t,64,_src,0)
  #define bupc_atomicU32_read_relaxed(_src)	_bupc_atomic_read_fixed(uint32_t,32,_src,0)
  #define bupc_atomicI32_read_relaxed(_src)	_bupc_atomic_read_fixed(int32_t,32,_src,0)
  #define bupc_atomicU64_read_strict(_src)	_bupc_atomic_read_fixed(uint64_t,64,_src,1)
  #define bupc_atomicI64_read_strict(_src)	_bupc_atomic_read_fixed(int64_t,64,_src,1)
  #define bupc_atomicU32_read_strict(_src)	_bupc_atomic_read_fixed(uint32_t,32,_src,1)
  #define bupc_atomicI32_read_strict(_src)	_bupc_atomic_read_fixed(int32_t,32,_src,1)
  #define bupc_atomicU64_read_private(_src)	_bupc_atomic_read_fixed_local(uint64_t,64,_src)
  #define bupc_atomicI64_read_private(_src)	_bupc_atomic_read_fixed_local(int64_t,64,_src)
  #define bupc_atomicU32_read_private(_src)	_bupc_atomic_read_fixed_local(uint32_t,32,_src)
  #define bupc_atomicI32_read_private(_src)	_bupc_atomic_read_fixed_local(int32_t,32,_src)
  #define bupc_atomicU64_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchadd,_dst,_op,0)
  #define bupc_atomicI64_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchadd,_dst,_op,0)
  #define bupc_atomicU32_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchadd,_dst,_op,0)
  #define bupc_atomicI32_fetchadd_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchadd,_dst,_op,0)
  #define bupc_atomicU64_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchand,_dst,_op,0)
  #define bupc_atomicI64_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchand,_dst,_op,0)
  #define bupc_atomicU32_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchand,_dst,_op,0)
  #define bupc_atomicI32_fetchand_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchand,_dst,_op,0)
  #define bupc_atomicU64_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchor,_dst,_op,0)
  #define bupc_atomicI64_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchor,_dst,_op,0)
  #define bupc_atomicU32_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchor,_dst,_op,0)
  #define bupc_atomicI32_fetchor_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchor,_dst,_op,0)
  #define bupc_atomicU64_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(U64,fetchxor,_dst,_op,0)
  #define bupc_atomicI64_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(I64,fetchxor,_dst,_op,0)
  #define bupc_atomicU32_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(U32,fetchxor,_dst,_op,0)
  #define bupc_atomicI32_fetchxor_relaxed(_dst,_op)	_bupc_atomic_1op(I32,fetchxor,_dst,_op,0)
  #define bupc_atomicU64_swap_relaxed(_dst,_op)	_bupc_atomic_1op(U64,swap,_dst,_op,0)
  #define bupc_atomicI64_swap_relaxed(_dst,_op)	_bupc_atomic_1op(I64,swap,_dst,_op,0)
  #define bupc_atomicU32_swap_relaxed(_dst,_op)	_bupc_atomic_1op(U32,swap,_dst,_op,0)
  #define bupc_atomicI32_swap_relaxed(_dst,_op)	_bupc_atomic_1op(I32,swap,_dst,_op,0)
  #define bupc_atomicU64_fetchnot_relaxed(_dst)	_bupc_atomic_0op(U64,fetchnot,_dst,0)
  #define bupc_atomicI64_fetchnot_relaxed(_dst)	_bupc_atomic_0op(I64,fetchnot,_dst,0)
  #define bupc_atomicU32_fetchnot_relaxed(_dst)	_bupc_atomic_0op(U32,fetchnot,_dst,0)
  #define bupc_atomicI32_fetchnot_relaxed(_dst)	_bupc_atomic_0op(I32,fetchnot,_dst,0)
  #define bupc_atomicU64_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U64,cswap,_dst,_op0,_op1,0)
  #define bupc_atomicI64_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I64,cswap,_dst,_op0,_op1,0)
  #define bupc_atomicU32_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U32,cswap,_dst,_op0,_op1,0)
  #define bupc_atomicI32_cswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I32,cswap,_dst,_op0,_op1,0)
  #define bupc_atomicU64_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U64,mswap,_dst,_op0,_op1,0)
  #define bupc_atomicI64_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I64,mswap,_dst,_op0,_op1,0)
  #define bupc_atomicU32_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(U32,mswap,_dst,_op0,_op1,0)
  #define bupc_atomicI32_mswap_relaxed(_dst,_op0,_op1)	_bupc_atomic_2op(I32,mswap,_dst,_op0,_op1,0)
  #define bupc_atomicU64_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchadd,_dst,_op,1)
  #define bupc_atomicI64_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchadd,_dst,_op,1)
  #define bupc_atomicU32_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchadd,_dst,_op,1)
  #define bupc_atomicI32_fetchadd_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchadd,_dst,_op,1)
  #define bupc_atomicU64_fetchand_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchand,_dst,_op,1)
  #define bupc_atomicI64_fetchand_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchand,_dst,_op,1)
  #define bupc_atomicU32_fetchand_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchand,_dst,_op,1)
  #define bupc_atomicI32_fetchand_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchand,_dst,_op,1)
  #define bupc_atomicU64_fetchor_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchor,_dst,_op,1)
  #define bupc_atomicI64_fetchor_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchor,_dst,_op,1)
  #define bupc_atomicU32_fetchor_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchor,_dst,_op,1)
  #define bupc_atomicI32_fetchor_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchor,_dst,_op,1)
  #define bupc_atomicU64_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(U64,fetchxor,_dst,_op,1)
  #define bupc_atomicI64_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(I64,fetchxor,_dst,_op,1)
  #define bupc_atomicU32_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(U32,fetchxor,_dst,_op,1)
  #define bupc_atomicI32_fetchxor_strict(_dst,_op)	_bupc_atomic_1op(I32,fetchxor,_dst,_op,1)
  #define bupc_atomicU64_swap_strict(_dst,_op)	_bupc_atomic_1op(U64,swap,_dst,_op,1)
  #define bupc_atomicI64_swap_strict(_dst,_op)	_bupc_atomic_1op(I64,swap,_dst,_op,1)
  #define bupc_atomicU32_swap_strict(_dst,_op)	_bupc_atomic_1op(U32,swap,_dst,_op,1)
  #define bupc_atomicI32_swap_strict(_dst,_op)	_bupc_atomic_1op(I32,swap,_dst,_op,1)
  #define bupc_atomicU64_fetchnot_strict(_dst)	_bupc_atomic_0op(U64,fetchnot,_dst,1)
  #define bupc_atomicI64_fetchnot_strict(_dst)	_bupc_atomic_0op(I64,fetchnot,_dst,1)
  #define bupc_atomicU32_fetchnot_strict(_dst)	_bupc_atomic_0op(U32,fetchnot,_dst,1)
  #define bupc_atomicI32_fetchnot_strict(_dst)	_bupc_atomic_0op(I32,fetchnot,_dst,1)
  #define bupc_atomicU64_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U64,cswap,_dst,_op0,_op1,1)
  #define bupc_atomicI64_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I64,cswap,_dst,_op0,_op1,1)
  #define bupc_atomicU32_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U32,cswap,_dst,_op0,_op1,1)
  #define bupc_atomicI32_cswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I32,cswap,_dst,_op0,_op1,1)
  #define bupc_atomicU64_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U64,mswap,_dst,_op0,_op1,1)
  #define bupc_atomicI64_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I64,mswap,_dst,_op0,_op1,1)
  #define bupc_atomicU32_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(U32,mswap,_dst,_op0,_op1,1)
  #define bupc_atomicI32_mswap_strict(_dst,_op0,_op1)	_bupc_atomic_2op(I32,mswap,_dst,_op0,_op1,1)
  #define bupc_atomicU64_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchadd,_dst,_op)
  #define bupc_atomicI64_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchadd,_dst,_op)
  #define bupc_atomicU32_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchadd,_dst,_op)
  #define bupc_atomicI32_fetchadd_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchadd,_dst,_op)
  #define bupc_atomicU64_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchand,_dst,_op)
  #define bupc_atomicI64_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchand,_dst,_op)
  #define bupc_atomicU32_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchand,_dst,_op)
  #define bupc_atomicI32_fetchand_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchand,_dst,_op)
  #define bupc_atomicU64_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchor,_dst,_op)
  #define bupc_atomicI64_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchor,_dst,_op)
  #define bupc_atomicU32_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchor,_dst,_op)
  #define bupc_atomicI32_fetchor_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchor,_dst,_op)
  #define bupc_atomicU64_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(U64,fetchxor,_dst,_op)
  #define bupc_atomicI64_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(I64,fetchxor,_dst,_op)
  #define bupc_atomicU32_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(U32,fetchxor,_dst,_op)
  #define bupc_atomicI32_fetchxor_private(_dst,_op)	_bupc_atomic_1op_local(I32,fetchxor,_dst,_op)
  #define bupc_atomicU64_swap_private(_dst,_op)	_bupc_atomic_1op_local(U64,swap,_dst,_op)
  #define bupc_atomicI64_swap_private(_dst,_op)	_bupc_atomic_1op_local(I64,swap,_dst,_op)
  #define bupc_atomicU32_swap_private(_dst,_op)	_bupc_atomic_1op_local(U32,swap,_dst,_op)
  #define bupc_atomicI32_swap_private(_dst,_op)	_bupc_atomic_1op_local(I32,swap,_dst,_op)
  #define bupc_atomicU64_fetchnot_private(_dst)	_bupc_atomic_0op_local(U64,fetchnot,_dst)
  #define bupc_atomicI64_fetchnot_private(_dst)	_bupc_atomic_0op_local(I64,fetchnot,_dst)
  #define bupc_atomicU32_fetchnot_private(_dst)	_bupc_atomic_0op_local(U32,fetchnot,_dst)
  #define bupc_atomicI32_fetchnot_private(_dst)	_bupc_atomic_0op_local(I32,fetchnot,_dst)
  #define bupc_atomicU64_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U64,cswap,_dst,_op0,_op1)
  #define bupc_atomicI64_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I64,cswap,_dst,_op0,_op1)
  #define bupc_atomicU32_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U32,cswap,_dst,_op0,_op1)
  #define bupc_atomicI32_cswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I32,cswap,_dst,_op0,_op1)
  #define bupc_atomicU64_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U64,mswap,_dst,_op0,_op1)
  #define bupc_atomicI64_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I64,mswap,_dst,_op0,_op1)
  #define bupc_atomicU32_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(U32,mswap,_dst,_op0,_op1)
  #define bupc_atomicI32_mswap_private(_dst,_op0,_op1)	_bupc_atomic_2op_local(I32,mswap,_dst,_op0,_op1)
#endif

#endif /* UPCR_ATOMIC_DEFS_H */
